/* 
  Copyright (c) 1997-2003 Gavin E. Crooks <gec@compbio.berkeley.edu> 
		     Univ. of California, Berkeley

  Permission is hereby granted, free of charge, to any person obtaining a 
  copy of this software and associated documentation files (the "Software"),
  to deal in the Software without restriction, including without limitation
  the rights to use, copy, modify, merge, publish, distribute, sublicense,
  and/or sell copies of the Software, and to permit persons to whom the
  Software is furnished to do so, subject to the following conditions:

  The above copyright notice and this permission notice shall be included
  in all copies or substantial portions of the Software.

  THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR 
  IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY, 
  FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
  AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER 
  LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
  OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN 
  THE SOFTWARE.

  (This is the MIT Open Source License, 
  http://www.opensource.org/licenses/mit-license.html)

*/


/* *******************************************************************
   Computational Biology Toolkit - Utilities - Unit Tests

   $Id: test_cbt_util.c,v 1.1 2003/08/19 03:55:47 gec Exp $
******************************************************************** */

#include <stdio.h>
#include <stdlib.h>
#include <assert.h>
#include <math.h>

#include "cbt_util.h"


void 
test_start(char * testname) {
  printf("TEST: %s\n", testname);
}

void 
test_argmax(void) {
  size_t i, argmax;
  int values1[] = {0,1,2,3,4,5,6,7,8,9,10};
  int values2[] = {0,-1,-2,-3,-4,-5,-6,-7,-8,-9,-10};

  test_start("argmax");

  for(i=0; i<11; i++) {
    argmax = cbt_argmax(i+1, values1);
    assert(argmax == i);
  }

  for(i=0; i<11; i++) {
    argmax = cbt_argmax(i, values2);
    assert(argmax == 0);
  }

}

void 
test_argmin(void) {
  size_t i;
  int values0[] = { 1,0,10};
  int values1[] = { 0,-10,-11,10};
  int values2[] = { 0, -1, -2, -3, -4, -5, -6, -7};

  test_start("argmin");

  assert( 0 == cbt_argmin(0, values0) );
  assert( 0 == cbt_argmin(1, values0) );
  assert( 1 == cbt_argmin(3, values0) );

  assert( 2 == cbt_argmin(4, values1) );

  for(i=0; i<8; i++) assert( i == cbt_argmin(i+1, values2) );
}

void 
test_argmin_dbl(void) {
  size_t i;
  double values0[] = { 0.1, 0.01, 0.02};
  double values1[] = { 0, -10.1, -11.2, 10.5};
  double values2[] = { 0, -1.1, -2.2, -3.3, -4.4, -5.5, -6.6, -7.7};

  test_start("argmin_dbl");

  assert( 0 == cbt_argmin_dbl(0, values0) );
  assert( 0 == cbt_argmin_dbl(1, values0) );
  assert( 1 == cbt_argmin_dbl(3, values0) );

  assert( 2 == cbt_argmin_dbl(4, values1) );

  for(i=0; i<8; i++) assert( i == cbt_argmin_dbl(i+1, values2) );
}

void
test_argmax_dbl(void) {
  size_t i;
  double values0[] = { 0.1, 1.01, 0.02};
  double values1[] = { 0, 10.1, 11.2, 10.5};

    double values2[] = { -10, 1.1, 2.2, 3.3, 4.4, 5.5, 6.6, 7.7};

  test_start("argmax_dbl");

  assert( 0 == cbt_argmax_dbl(0, values0) );
  assert( 0 == cbt_argmax_dbl(1, values0) );
  assert( 1 == cbt_argmax_dbl(3, values0) );

  assert( 2 == cbt_argmax_dbl(4, values1) );

  for(i=0; i<8; i++) assert( i == cbt_argmax_dbl(i+1, values2) );
}

void
test_normalize(void) {
  double values0[] = { 0.1, 1.01, 0.02};
  double values2[] = { -10, 1.1, 2.2, 3.3, 4.4, 5.5, 6.6, 7.7};
  double total;

  test_start("normalize");

  total = cbt_normalize(3, values0);
  assert(fabs(total-1.13) < 16*__CBT_DBL_EPSILON__); 
  total = cbt_normalize(3, values0);
  assert(fabs(total-1.0) < 16*__CBT_DBL_EPSILON__); 

  total = cbt_normalize(8, values2);
  assert(fabs(total-20.8) < 16*__CBT_DBL_EPSILON__); 
}



void 
test_ran_clock(void) {
  unsigned long int r;
  test_start("ran_clock");
  r = cbt_ran_clock();
}

void
test_die(void) {
  test_start("die: NOT IMPLEMENTED");
  // NOT IMPLEMENTED
}

void 
test_constants(void) {
  test_start("constants");
  assert(fabs( cbt_bits_per_nat - (1./log(2))) < 16.*__CBT_DBL_EPSILON__); 
}


void
test_mean(void) {
  double values0[] = { 0.5, 0.5};
  double values2[] = { -10., 1.1, 2.2, 3.3, 4.4, 5.5, 6.6, 7.7};
  double res;

  test_start("mean");

  res = cbt_mean(2, values0);
  assert(fabs(res-0.5) < 16*__CBT_DBL_EPSILON__);
 
  res = cbt_mean(8, values2);
  assert(fabs(res-(20.8/8.)) < 16*__CBT_DBL_EPSILON__); 

  res = cbt_normalize(8, values2);
  res = cbt_mean(8, values2);
  assert(fabs(res-(1./8.)) < 16*__CBT_DBL_EPSILON__); 
}

void
test_variance(void) {
  test_start("variance: NOT IMPLEMENTED");
}

void
test_stddev(void) {
  double values2[] = { -10., 1.1, 2.2, 3.3, 4.4, 5.5, 6.6, 7.7};
  double s,v;

  test_start("stddev");

  v = cbt_variance(8, values2);
  s = cbt_stddev(8, values2);
  assert( fabs(sqrt(v)-s) < 16*__CBT_DBL_EPSILON__);

}

void
test_entropy(void) {
  double data[] = { 1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1};
  double res;

  test_start("entropy");
  cbt_normalize(16, data);
  res = cbt_entropy(16,data);

  assert( fabs( res - 4.) < 16*__CBT_DBL_EPSILON__);
}









int main(void) {
  test_argmax();
  test_argmin();
  test_argmax_dbl();
  test_argmin_dbl();

  test_ran_clock();
  test_die();
  test_constants();

  test_normalize();
  test_mean();
  test_variance();
  test_stddev();
  test_entropy();
  

  // printf(" Tests Passed\n");

  exit(0);
}
